from math import *
from random import *
import matplotlib.pyplot as plt
import numpy as np

#1-dimensional symmetric random walk metropolis algorithm

#Denote our target distribution is pi(x)
#Suppose in case 1, target distribution is exponential(1) distribution.
def pi_1(x):
    if x < 0:
        return 0
    else:
        return e**(-x)
    
#A way to sample from the target distribution is called the symmetric random
#walk metropolis algorithm. We need to determine the optimal scaling sigma.

#Suppose we want to run the algorithm for n steps, here we take n=500 for now.
#For the scaling we take sigma = 1 for now.
def sample():
    
    X = 0 #Let us start with an initial state X
    reject_times = 0 #want to know how many times the proposed value y is rejected
    i = 1
    while i <= 500:
        Z = normalvariate(0,300) #Assume Python can generate iid normal random variables
        #and the proposal distribution is symmetric q(x,y) = q(y,x)
        Y_pro = X + Z
        #Determine acceptance proabability
        if pi_1(X) == 0:
            a = 1
        else:
            a = min(pi_1(Y_pro)/float(pi_1(X)),1)
        #Use this probability to accept/reject:
        k = random()
        if k < a:
            X = Y_pro
        else:
            X = X #proposal is rejected
            reject_times += 1
        i += 1
    return X #, str(reject_times/500.0) + " is the rejection rate."
    
#We would like to know whether the sampling distribution is a good approximation
#of the exponential(1) distribution.

#get a sample of size 200 and compute the empirical distribution to compare with
#the cumulative distribution of exponential(1)
list0 = []
i = 1
while i <= 500:
    a = sample()
    list0.append(a)
    i += 1

'''One interesting observation here about the simulation is that don't start
with an initial state that is not in the state space of the target distribution
I made state 0 with density 0 for exponential(1) distribution and I started at
state 0. In the end I got some mighty strange simulations -- negative values.'''

#first we constructed the cdf for exponential(1)in the interval [0,5], plot it in red
def f(x):
    return 1-exp(-x)
a=np.vectorize(f)
x_val = np.arange(0.0,5.0,0.01)
y_val = a(x_val)

plt.plot(x_val,y_val, 'r')

#Compute the empirical distribution in the interval [0,5], plot in green
y_empirical = []
for element in x_val:
    count  = 0
    for ele in list0:
        if ele <= element:
            count += 1
    proportion = count/500.0
    y_empirical.append(proportion)
plt.plot(x_val,y_empirical, 'g')
plt.show()    


            
    
