'''Code for Case4'''

from numpy import *
from scipy import *
from math import *
from random import *
from statistics import *
import matplotlib.pyplot as plt

''' 3 point process with a specific density function on [0,1.5] * [0,1]'''

#initial position
x_1 = 1.5*random()
y_1 = random()
x_2 = 1.5*random()
y_2 = random()
x_3 = 1.5*random()
y_3 = random()

ini_pos = array([x_1,y_1,x_2,y_2,x_3,y_3],float)

#Now define pdf
def h(x, A,B,C):
    #x is the array for positions of the 3 particles
    sum_dis = float(sqrt((x[0]-x[2])**2+(x[1]-x[3])**2) + sqrt((x[0]-x[4])**2+(x[1]-x[5])**2) + sqrt((x[2]-x[4])**2+(x[3]-x[5])**2))
    sum_x = float(x[0]+x[2]+x[4])
    value = A*sum_dis + 0.04*B*(1/float(sqrt((x[0]-x[2])**2+(x[1]-x[3])**2))+1/float(sqrt((x[0]-x[4])**2+(x[1]-x[5])**2))+1/float(sqrt((x[2]-x[4])**2+(x[3]-x[5])**2))) + C*sum_x
    return value

#def pi(x,A,B,C):
    ##target distribution up to a constant
    #if 0<=x[0]<=1.5 and 0<=x[2]<=1.5 and 0<=x[4]<=1.5 and 0<=x[1]<=1 and 0<=x[3]<=1 and 0<=x[5]<=1:
        ##restricting to pink region
        #return exp(-h(x,A,B,C))
    #else:
        #return 0
    
#We commented out the definition of pdf because we are doing the rejection/acceptance on log scale
#to avoid numerical errors
'''Metropolis within Gibbs Adaptive Algorithm to Target Acceptance Rate'''

def run(step,A,B,C,rate_set):
    #rate set is the target acceptance rate
    x = ini_pos
    reject_times = 0
    accept_times = 0
    
    x_1_pos = []
    y_1_pos = []
    time = []
    length = 0.1
    
    i = 0
    while i <= step:
        time.append(i)
        x_1_pos.append(x[0])
        y_1_pos.append(x[1])
        
        
        dummy = [0,0,0,0,0,0]
        #pick a point at random
        a = randint(1,3)
        r1 = uniform(-length,length)
        r2 = uniform(-length,length)
        x_pos = a*2-2
        dummy[x_pos] = r1
        dummy[x_pos+1] = r2
        
        increment = array(dummy)
        proposal = x + increment 
        
        if (0<=proposal[0]<=1.5 and 0<=proposal[2]<=1.5 and 0<=proposal[4]<=1.5 and 0<=proposal[1]<=1 and 0<=proposal[3]<=1 and 0<=proposal[5]<=1):
            accept = h(x,A,B,C) - h(proposal,A,B,C)
            
            k = random()
            if log(k) < accept:
                x = proposal
                accept_times += 1
            else:
                x = x #proposal is rejected
                reject_times += 1
        else:
            x = x #proposal is rejected
            reject_times += 1
        #Make adapatation every 50 runs satisfying the diminishing condition to assure ergodicity
        if i%50 == 0:
            acc_rate = 1 - reject_times/float(i+1)
            if acc_rate > rate_set:
                length += min(0.01, 1/float(i+1))
            elif acc_rate < rate_set:
                length -= min(0.01, 1/float(i+1))
        i +=1
    return x_1_pos,y_1_pos,time,1-(reject_times/float(step)), accept_times, reject_times



#To estimate Var(1/n \sum g(Xi)) where g is taken as the x coordinate of the first particle
def get_variance(step,A,B,C, rate):
    k = 1
    mean_0 = []
    while k <= 20:
        a = run(step,A,B,C, rate)
        mean_0.append(mean(a[0]))
        k += 1
    return (std(mean_0))**2

def get_corr_x0_xi(l,i):
    x=[]
    y=[]
    k = 0
    while k + i < len(l):
        x.append(l[k])
        y.append(l[k+i])
        k += 1
    return correlation(x,y)

#To get the graph of auto correlation time
def get_corr_graph(l,step):
    a = []
    b = []
    i = 1
    while i <= step:
        a.append(get_corr_x0_xi(l,i))
        b.append(i)
        i += 1
    return a,b



def get_iact(step,A,B,C,rate):
    '''Here g is taken as the first coordinate of the vector'''
    
    x1_final = run(step,A,B,C,rate)[0]
    sum = 0
    i = 1
    k = 0
    while i <= step and k <= 1:
        q = get_corr_x0_xi(x1_final,i)
        if q < 0:
            k += 1
        sum += q
        i += 1
    return 1 + 2*sum, i

#set the cutoff value manually
def get_iact_manual(step,A,B,C,cutoff, rate):
    '''Here g is taken as the first coordinate of the vector'''
    
    x1_final = run(step,A,B,C,rate)[0]
    sum = 0
    i = 1
    while i <= cutoff:
        q = get_corr_x0_xi(x1_final,i)
        sum += q
        i += 1
    return 1 + 2*sum, i

def get_withop(step,l):
    '''Here g is taken as the first coordinate of the vector'''
    
    x1_final = l
    sum = 0
    i = 1
    k = 0
    while i <= step and k <= 1:
        q = get_corr_x0_xi(x1_final,i)
        if q < 0:
            k += 1
        sum += q
        i += 1
    return 1 + 2*sum, i
        
#a = run(300000,200,200,0,0.234)
#taug = get_withop(50000,a[0])
#var1 = std(a[0])**2
#var2 = get_variance(300000,200,200,0,0.234)
#print "tau g", taug[0]
#print "lag", taug[1]
#print "var(g(X))", var1
#print "other variance", var2
#print "ratio", 300000*var2/float(taug[0])


#c = get_corr_graph(a[0],50000)
#plt.plot(c[1],c[0], "b")
#plt.show() 


